@@ -9,35 +9,41 @@ if ($ARGV[0] eq "--debug" || $ARGV[0] eq "-debug") {
 	}
 
 # See what needs doing
+&flush_package_caches();
+&clear_repository_cache();
 @todo = &list_possible_updates();
-foreach $a (@todo) {
-	$a->{'level'} = $a->{'security'} ? 1 : 2;
-	}
 
 # Install packages that are needed
 $tellcount = 0;
 %already = ( );
 &start_update_progress([ map { $_->{'name'} } @todo ]);
+$icount = 0;
 foreach $t (@todo) {
 	next if ($already{$t->{'update'}});
-	if ($t->{'level'} <= $config{'sched_action'}) {
+	my $umsg = $t->{'security'} ? "security update" : "update";
+	my $upfx = $t->{'security'} ? "A" : "An";
+	if ($config{'sched_action'} == 2 ||
+	    $config{'sched_action'} == 1 && $t->{'security'}) {
 		# Can install
-		$body .= "An update to $t->{'name'} from $t->{'oldversion'} to $t->{'version'} is needed.\n";
+		$body .= "$upfx $umsg to $t->{'name'} from $t->{'oldversion'} to $t->{'version'} is needed.\n";
+		$icount++;
 		($out, $done) = &capture_function_output(
 				  \&package_install, $t->{'update'});
 		if (@$done) {
-			$body .= "This update has been successfully installed.\n\n";
+			$body .= "This $umsg has been successfully installed.\n\n";
 			}
 		else {
-			$body .= "However, this update could not be installed! Try the update manually\nusing the Package Updates module.\n\n";
+			$body .= "However, this $usmg could not be installed! Try the update manually\nusing the Package Updates module.\n\n";
 			}
 		foreach $p (@$done) {
 			$already{$p}++;
 			}
 		}
-	else {
+	elsif ($config{'sched_action'} == 1 ||
+	       $config{'sched_action'} == 0 ||
+	       $config{'sched_action'} == -1 && $t->{'security'}) {
 		# Just tell the user about it
-		$body .= "An update to $t->{'name'} from $t->{'oldversion'} to $t->{'version'} is available.\n\n";
+		$body .= "$upfx $umsg to $t->{'name'} from $t->{'oldversion'} to $t->{'version'} is available.\n\n";
 		$tellcount++;
 		}
 	}
@@ -50,12 +56,13 @@ if ($tellcount) {
 	}
 
 # Email the admin
-if ($config{'sched_email'} && $body) {
+$emailto = $config{'sched_email'} || $gconfig{'webmin_email_to'};
+if ($emailto && $body) {
 	&foreign_require("mailboxes", "mailboxes-lib.pl");
 	my $from = &mailboxes::get_from_address();
 	my $mail = { 'headers' =>
 			[ [ 'From', $from ],
-			  [ 'To', $config{'sched_email'} ],
+			  [ 'To', $emailto ],
 			  [ 'Subject', "Package updates on ".
 				       &get_system_hostname() ] ],
 			'attach' =>
@@ -67,3 +74,7 @@ if ($config{'sched_email'} && $body) {
 		}
 	}
 
+# Log the update, if anything was installed
+if ($icount) {
+	&webmin_log("schedup", "packages", $icount);
+	}
